@@ -1,7 +1,19 @@
 #include <string.h>
 #include <stdlib.h>
 #include "everything.h"
-#include <gio/gio.h>
+
+static gboolean abort_on_error = TRUE;
+
+#define ASSERT_VALUE(condition)  \
+  if (abort_on_error)             \
+    g_assert (condition);         \
+  else                            \
+    g_warn_if_fail (condition);   \
+
+void set_abort_on_error (gboolean in)
+{
+  abort_on_error = in;
+}
 
 /* basic types */
 gboolean test_boolean (gboolean in)
@@ -9,6 +21,18 @@ gboolean test_boolean (gboolean in)
   return in;
 }
 
+gboolean test_boolean_true (gboolean in)
+{
+  ASSERT_VALUE (in == TRUE);
+  return in;
+}
+
+gboolean test_boolean_false (gboolean in)
+{
+  ASSERT_VALUE (in == FALSE);
+  return in;
+}
+
 gint8 test_int8 (gint8 in)
 {
   return in;
@@ -184,6 +208,84 @@ const GValue *test_value_return(int i) {
   return &value;
 }
 
+/************************************************************************/
+/* foreign structs */
+
+/**
+ * test_cairo_context_full_return:
+ * Returns: (transfer full):
+ */
+cairo_t *
+test_cairo_context_full_return (void)
+{
+  cairo_surface_t *surface;
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
+  return cairo_create (surface);
+}
+
+/**
+ * test_cairo_context_none_in:
+ * @context: (transfer none):
+ */
+void
+test_cairo_context_none_in (cairo_t *context)
+{
+  cairo_surface_t *surface = cairo_get_target (context);
+
+  g_assert (cairo_image_surface_get_format (surface) == CAIRO_FORMAT_ARGB32);
+  g_assert (cairo_image_surface_get_width (surface) == 10);
+  g_assert (cairo_image_surface_get_height (surface) == 10);
+}
+
+
+/**
+ * test_cairo_surface_none_return:
+ * Returns: (transfer none):
+ */
+cairo_surface_t *
+test_cairo_surface_none_return (void)
+{
+  static cairo_surface_t *surface;
+
+  if (surface == NULL) {
+    surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
+  }
+
+  return surface;
+}
+
+/**
+ * test_cairo_surface_full_return:
+ * Returns: (transfer full):
+ */
+cairo_surface_t *
+test_cairo_surface_full_return (void)
+{
+  return cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
+}
+
+/**
+ * test_cairo_surface_none_in:
+ * @surface: (transfer none):
+ */
+void
+test_cairo_surface_none_in (cairo_surface_t *surface)
+{
+  g_assert (cairo_image_surface_get_format (surface) == CAIRO_FORMAT_ARGB32);
+  g_assert (cairo_image_surface_get_width (surface) == 10);
+  g_assert (cairo_image_surface_get_height (surface) == 10);
+}
+
+/**
+ * test_cairo_surface_full_out:
+ * @surface: (out) (transfer full):
+ */
+void
+test_cairo_surface_full_out (cairo_surface_t **surface)
+{
+  *surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
+}
+
 
 /************************************************************************/
 /* utf8 */
@@ -313,6 +415,24 @@ test_utf8_out_nonconst_return (char **out)
   return g_strdup ("first");
 }
 
+/**
+ * test_utf8_null_in:
+ * @in: (allow-none):
+ */
+void test_utf8_null_in (char *in)
+{
+  g_assert (in == NULL);
+}
+
+/**
+ * test_utf8_null_out:
+ * @char_out: (allow-none) (out):
+ */
+void test_utf8_null_out (char **char_out)
+{
+  *char_out = NULL;
+}
+
 
 /* non-basic-types */
 
@@ -630,6 +750,27 @@ test_array_int_none_out(int *len) {
   return result;
 }
 
+/**
+ * test_array_int_null_in:
+ * @arr: (array length=len) (allow-none):
+ * @len: length
+ */
+void test_array_int_null_in (int *arr, int len)
+{
+  g_assert (arr == NULL);
+}
+
+/**
+ * test_array_int_null_out:
+ * @arr: (out) (array length=len) (allow-none):
+ * @len: (out) : length
+ */
+void test_array_int_null_out (int **arr, int *len)
+{
+  *arr = NULL;
+  *len = 0;
+}
+
 /* interface */
 
 /************************************************************************/
@@ -754,6 +895,25 @@ void test_glist_everything_in (GList *in)
   test_glist_free (in);
 }
 
+/**
+ * test_glist_null_in:
+ * @in: (element-type utf8) (allow-none):
+ */
+void test_glist_null_in (GSList *in)
+{
+  g_assert (in == NULL);
+}
+
+/**
+ * test_glist_null_out:
+ * @out_list: (out) (element-type utf8) (allow-none):
+ */
+void test_glist_null_out (GSList **out_list)
+{
+  *out_list = NULL;
+}
+
+
 /************************************************************************/
 /* GSList */
 
@@ -876,6 +1036,24 @@ void test_gslist_everything_in (GSList *in)
   test_gslist_free (in);
 }
 
+/**
+ * test_gslist_null_in:
+ * @in: (element-type utf8) (allow-none):
+ */
+void test_gslist_null_in (GSList *in)
+{
+  g_assert (in == NULL);
+}
+
+/**
+ * test_gslist_null_out:
+ * @out_list: (out) (element-type utf8) (allow-none):
+ */
+void test_gslist_null_out (GSList **out_list)
+{
+  *out_list = NULL;
+}
+
 /************************************************************************/
 /* GHash */
 
@@ -998,6 +1176,15 @@ void test_ghash_null_in (const GHashTable *in)
   g_assert (in == NULL);
 }
 
+/**
+ * test_ghash_null_out:
+ * @out: (element-type utf8 utf8) (allow-none) (out):
+ */
+void test_ghash_null_out (const GHashTable **out)
+{
+  *out = NULL;
+}
+
 /**
  * test_ghash_nothing_in:
  * @in: (element-type utf8 utf8):
@@ -1644,6 +1831,39 @@ test_callback (TestCallback callback)
     return 0;
 }
 
+/**
+ * test_multi_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+int
+test_multi_callback (TestCallback callback)
+{
+    int sum = 0;
+    if (callback != NULL) {
+        sum += callback();
+        sum += callback();
+    }
+
+    return sum;
+}
+
+
+
+/**
+ * test_simple_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+test_simple_callback (TestSimpleCallback callback)
+{
+    if (callback != NULL)
+        callback();
+
+    return;
+}
+
 /**
  * test_callback_user_data:
  * @callback: (scope call):
@@ -1758,22 +1978,59 @@ test_callback_thaw_async (void)
   return retval;
 }
 
-/**
- * test_callback_infinite:
- * @callback: (scope infinite):
- *
- * Infinite - callback persists forever.
- **/
-
-static GSList *infinite_callbacks = NULL;
-
-int
-test_callback_infinite (TestCallbackUserData callback,
-                       gpointer user_data)
+void
+test_async_ready_callback (GAsyncReadyCallback callback)
 {
-  infinite_callbacks = g_slist_prepend(infinite_callbacks, callback);
+  GSimpleAsyncResult *result = g_simple_async_result_new (NULL, callback, NULL,
+    test_async_ready_callback);
+  g_simple_async_result_complete_in_idle (result);
+}
 
-  return callback(user_data);
+/**
+ * test_obj_instance_method_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+test_obj_instance_method_callback (TestObj *obj, TestCallback callback)
+{
+    if (callback != NULL)
+        callback();
+}
+
+/**
+ * test_obj_static_method_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+test_obj_static_method_callback (TestCallback callback)
+{
+    if (callback != NULL)
+        callback();
+}
+
+/**
+ * test_obj_new_callback:
+ * @callback: (scope notified):
+ *
+ **/
+TestObj *
+test_obj_new_callback (TestCallbackUserData callback, gpointer user_data,
+                       GDestroyNotify notify)
+{
+  CallbackInfo *info;
+
+  callback(user_data);
+
+  info = g_slice_new(CallbackInfo);
+  info->callback = callback;
+  info->notify = notify;
+  info->user_data = user_data;
+
+  notified_callbacks = g_slist_prepend(notified_callbacks, info);
+
+  return g_object_new (TEST_TYPE_OBJ, NULL);
 }
 
 /* interface */
