@@ -1,11 +1,17 @@
 #ifndef __GITESTTYPES_H__
 #define __GITESTTYPES_H__
 
+#include <cairo/cairo.h>
 #include <glib-object.h>
+#include <gio/gio.h>
 #include <time.h>
 
+void set_abort_on_error (gboolean abort_on_error);
+
 /* basic types */
 gboolean test_boolean (gboolean in);
+gboolean test_boolean_true (gboolean in);
+gboolean test_boolean_false (gboolean in);
 gint8 test_int8 (gint8 in);
 guint8 test_uint8 (guint8 in);
 gint16 test_int16 (gint16 in);
@@ -35,6 +41,8 @@ void test_utf8_const_in (const char *in);
 void test_utf8_out (char **out);
 void test_utf8_inout (char **inout);
 GSList *test_filename_return (void);
+void test_utf8_null_in (char *in);
+void test_utf8_null_out (char **char_out);
 
 /* in arguments after out arguments */
 void test_int_out_utf8 (int *length, const char *in);
@@ -69,6 +77,8 @@ gboolean test_strv_in_container (char **arr);
 
 int *test_array_int_full_out(int *len);
 int *test_array_int_none_out(int *len);
+void test_array_int_null_in (int *arr, int len);
+void test_array_int_null_out (int **arr, int *len);
 
 /* interface */
 /* GList */
@@ -80,6 +90,8 @@ void test_glist_nothing_in (const GList *in);
 void test_glist_nothing_in2 (GList *in);
 void test_glist_container_in (GList *in);
 void test_glist_everything_in (GList *in);
+void test_glist_null_in(GSList *in);
+void test_glist_null_out(GSList **out_list);
 void test_glist_free (GList *in);
 
 /* GSList */
@@ -91,6 +103,8 @@ void test_gslist_nothing_in (const GSList *in);
 void test_gslist_nothing_in2 (GSList *in);
 void test_gslist_container_in (GSList *in);
 void test_gslist_everything_in (GSList *in);
+void test_gslist_null_in(GSList *in);
+void test_gslist_null_out(GSList **out_list);
 void test_gslist_free (GSList *in);
 
 /* GHashTable */
@@ -100,6 +114,7 @@ GHashTable *test_ghash_nothing_return2 (void);
 GHashTable *test_ghash_container_return (void);
 GHashTable *test_ghash_everything_return (void);
 void test_ghash_null_in (const GHashTable *in);
+void test_ghash_null_out (const GHashTable **out);
 void test_ghash_nothing_in (const GHashTable *in);
 void test_ghash_nothing_in2 (GHashTable *in);
 void test_ghash_container_in (GHashTable *in);
@@ -118,6 +133,16 @@ int test_closure_one_arg (GClosure *closure, int arg);
 int test_int_value_arg(const GValue *v);
 const GValue *test_value_return(int i);
 
+/* foreign structs */
+cairo_t *test_cairo_context_full_return (void);
+void test_cairo_context_none_in (cairo_t *context);
+
+cairo_surface_t *test_cairo_surface_none_return (void);
+cairo_surface_t *test_cairo_surface_full_return (void);
+void test_cairo_surface_none_in (cairo_surface_t *surface);
+void test_cairo_surface_full_out (cairo_surface_t **surface);
+
+
 /* enums / flags */
 
 typedef enum
@@ -216,6 +241,10 @@ struct _TestBoxed
 
 GType      test_boxed_get_type (void);
 TestBoxed *test_boxed_new      (void);
+TestBoxed *test_boxed_new_alternative_constructor1  (int i);
+TestBoxed *test_boxed_new_alternative_constructor2  (int i, int j);
+TestBoxed *test_boxed_new_alternative_constructor3  (char *s);
+
 TestBoxed *test_boxed_copy     (TestBoxed *boxed);
 gboolean   test_boxed_equals   (TestBoxed *boxed,
 				TestBoxed *other);
@@ -301,10 +330,13 @@ void        test_sub_obj_unset_bare (TestSubObj *obj);
 int         test_sub_obj_instance_method (TestSubObj *obj);
 
 /* callback */
-typedef int (*TestCallback) ();
+typedef void (*TestSimpleCallback) (void);
+typedef int (*TestCallback) (void);
 typedef int (*TestCallbackUserData) (gpointer user_data);
 
+void test_simple_callback (TestSimpleCallback callback);
 int test_callback (TestCallback callback);
+int test_multi_callback (TestCallback callback);
 int test_callback_user_data (TestCallbackUserData callback,
                              gpointer user_data);
 int test_callback_destroy_notify (TestCallbackUserData callback,
@@ -315,8 +347,13 @@ int test_callback_thaw_notifications (void);
 void test_callback_async (TestCallbackUserData callback, gpointer user_data);
 int test_callback_thaw_async (void);
 
-int test_callback_infinite (TestCallbackUserData callback,
-                           gpointer user_data);
+void test_async_ready_callback (GAsyncReadyCallback callback);
+
+void test_obj_instance_method_callback (TestObj *obj, TestCallback callback);
+void test_obj_static_method_callback (TestCallback callback);
+TestObj *test_obj_new_callback (TestCallbackUserData callback,
+                                gpointer user_data,
+                                GDestroyNotify notify);
 
 /* interface */
 #define TEST_TYPE_INTERFACE              (test_interface_get_type ())
