@@ -31,6 +31,9 @@ from .ast import (Array, Bitfield, Callback, Class, Enum, Field, Function,
                   PARAM_DIRECTION_INOUT,
                   PARAM_DIRECTION_IN,
                   PARAM_DIRECTION_OUT,
+                  PARAM_SCOPE_CALL,
+                  PARAM_SCOPE_ASYNC,
+                  PARAM_SCOPE_NOTIFIED,
                   PARAM_TRANSFER_NONE,
                   PARAM_TRANSFER_CONTAINER,
                   PARAM_TRANSFER_FULL,
@@ -496,7 +499,22 @@ class AnnotationApplier(object):
                 return
             field.type.name = self._transformer.resolve_type_name(t.one())
 
+    def _check_arg_annotations(self, parent, params, block):
+        if block is None:
+            return
+        for tag in block.tags.keys():
+            if tag == TAG_RETURNS:
+                continue
+            for param in params:
+                if param.name == tag:
+                    break
+            else:
+                return
+                #print 'WARNING: annotation for "%s" refers to unknown ' \
+                #        'argument "%s"' % (parent.name, tag)
+
     def _parse_params(self, parent, params, block):
+        self._check_arg_annotations(parent, params, block)
         for param in params:
             tag = self._get_tag(block, param.name)
             self._parse_param(parent, param, tag)
@@ -505,12 +523,28 @@ class AnnotationApplier(object):
         tag = self._get_tag(block, TAG_RETURNS)
         self._parse_param_ret_common(parent, return_, tag)
 
+    def _get_parameter_index(self, parent, param_name, location_name):
+        index = parent.get_parameter_index(param_name)
+        if index is None:
+            raise InvalidAnnotationError(
+                "can't find parameter %s referenced by parameter %s of %r"
+                % (param_name, location_name, parent.name))
+
+        return index
+
     def _parse_param(self, parent, param, tag):
         options = getattr(tag, 'options', {})
         if isinstance(parent, Function):
             scope = options.get(OPT_SCOPE)
             if scope:
                 param.scope = scope.one()
+                if param.scope not in [PARAM_SCOPE_CALL,
+                                       PARAM_SCOPE_ASYNC,
+                                       PARAM_SCOPE_NOTIFIED]:
+                    raise InvalidAnnotationError(
+                        "scope for %s of %r is invalid (%r), must be one of "
+                        "call, async, notified."
+                        % (param.name, parent.name, param.scope))
                 param.transfer = PARAM_TRANSFER_NONE
             elif (param.type.ctype == 'GAsyncReadyCallback' or
                   param.type.name == 'Gio.AsyncReadyCallback'):
@@ -519,14 +553,22 @@ class AnnotationApplier(object):
 
             destroy = options.get(OPT_DESTROY)
             if destroy:
-                param.destroy_index = parent.get_parameter_index(destroy.one())
+                param.destroy_index = self._get_parameter_index(parent,
+                                                                destroy.one(),
+                                                                param.name)
                 self._fixup_param_destroy(parent, param)
             closure = options.get(OPT_CLOSURE)
             if closure:
-                param.closure_index = parent.get_parameter_index(closure.one())
+                param.closure_index = self._get_parameter_index(parent,
+                                                                closure.one(),
+                                                                param.name)
                 self._fixup_param_closure(parent, param)
         if isinstance(parent, Callback):
             if OPT_CLOSURE in options:
+                # For callbacks, (closure) appears without an
+                # argument, and tags a parameter that is a closure. We
+                # represent it (weirdly) in the gir and typelib by
+                # setting param.closure_index to its own index.
                 param.closure_index = parent.get_parameter_index(param.name)
                 self._fixup_param_closure(parent, param)
 
@@ -535,12 +577,12 @@ class AnnotationApplier(object):
     def _fixup_param_destroy(self, parent, param):
         for p in parent.parameters:
             if p is not param and p.destroy_index == param.destroy_index:
-                p.destroy_index = None
+                p.destroy_index = -1
 
     def _fixup_param_closure(self, parent, param):
         for p in parent.parameters:
             if p is not param and p.closure_index == param.closure_index:
-                p.closure_index = None
+                p.closure_index = -1
 
     def _parse_param_ret_common(self, parent, node, tag):
         options = getattr(tag, 'options', {})
@@ -590,6 +632,12 @@ class AnnotationApplier(object):
         has_element_type = OPT_ELEMENT_TYPE in options
         has_array = OPT_ARRAY in options
 
+        if not has_array:
+            has_array = \
+                node.type.name in ['GLib.Array', 'GLib.PtrArray',
+                                   'GLib.ByteArray'] or \
+                node.type.ctype in ['GArray*', 'GPtrArray*', 'GByteArray*']
+
         # FIXME: This is a hack :-(
         if (not isinstance(node, Field) and
             (not has_element_type and
@@ -615,13 +663,25 @@ class AnnotationApplier(object):
         else:
             array_values = {}
 
+        is_g_array = node.type.ctype.startswith('GArray*') or \
+                     node.type.ctype.startswith('GPtrArray*') or \
+                     node.type.ctype.startswith('GByteArray*')
+
         element_type = options.get(OPT_ELEMENT_TYPE)
         if element_type is not None:
             element_type_node = self._resolve(element_type.one())
         else:
-            element_type_node = Type(node.type.name) # erase ctype
+            if is_g_array:
+                element_type_node = None
+            else:
+                element_type_node = Type(node.type.name) # erase ctype
 
-        container_type = Array(node.type.ctype,
+        if is_g_array:
+            type_name = node.type.name
+        else:
+            type_name = None
+
+        container_type = Array(type_name, node.type.ctype,
                                element_type_node)
         container_type.is_const = node.type.is_const
         if OPT_ARRAY_ZERO_TERMINATED in array_values:
@@ -629,7 +689,7 @@ class AnnotationApplier(object):
                 OPT_ARRAY_ZERO_TERMINATED) == '1'
         length = array_values.get(OPT_ARRAY_LENGTH)
         if length is not None:
-            param_index = parent.get_parameter_index(length)
+            param_index = self._get_parameter_index(parent, length, node.name)
             container_type.length_param_index = param_index
             # For in parameters we're incorrectly deferring
             # char/unsigned char to utf8 when a length annotation
