@@ -82,6 +82,10 @@ PARAM_DIRECTION_IN = 'in'
 PARAM_DIRECTION_OUT = 'out'
 PARAM_DIRECTION_INOUT = 'inout'
 
+PARAM_SCOPE_CALL = 'call'
+PARAM_SCOPE_ASYNC = 'async'
+PARAM_SCOPE_NOTIFIED = 'notified'
+
 PARAM_TRANSFER_NONE = 'none'
 PARAM_TRANSFER_CONTAINER = 'container'
 PARAM_TRANSFER_FULL = 'full'
@@ -111,11 +115,27 @@ type_names['char*'] = TYPE_STRING
 type_names['void*'] = TYPE_ANY
 type_names['void'] = TYPE_NONE
 
-# C unix types
-type_names['off_t'] = TYPE_SIZET
-type_names['pid_t'] = TYPE_INT
+# Unix types that we special case here (and that have their own introspection
+# type tag) because of wide use in GLib.
 type_names['size_t'] = TYPE_SIZET
 type_names['ssize_t'] = TYPE_SSIZET
+
+# One off C unix type definitions; note some of these may be GNU Libc
+# specific.  If someone is actually bitten by this, feel free to do
+# the required configure goop to determine their size and replace
+# here.
+#
+# We don't want to encourage people to use these in their APIs because
+# they compromise the platform-independence that GLib gives you.
+# These are here mostly to avoid blowing when random platform-specific
+# methods are added under #ifdefs inside GLib itself.  We could just (skip)
+# the relevant methods, but on the other hand, since these types are just
+# integers it's easy enough to expand them.
+type_names['off_t'] = TYPE_SIZET
+type_names['pid_t'] = TYPE_INT
+type_names['uid_t'] = TYPE_UINT
+type_names['gid_t'] = TYPE_UINT
+type_names['dev_t'] = TYPE_INT
 type_names['socklen_t'] = TYPE_INT32
 
 # Obj-C
@@ -152,6 +172,7 @@ class Node(object):
     def __init__(self, name=None):
         self.name = name
         self.attributes = [] # (key, value)*
+        self.skip = False
         self.deprecated = None
         self.deprecated_version = None
         self.version = None
@@ -274,8 +295,10 @@ class Varargs(Type):
 
 class Array(Type):
 
-    def __init__(self, ctype, element_type):
-        Type.__init__(self, '<carray>', ctype)
+    def __init__(self, name, ctype, element_type):
+        if name is None:
+            name = '<carray>'
+        Type.__init__(self, name, ctype)
         self.element_type = element_type
         self.zeroterminated = True
         self.length_param_index = -1
